@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Periodic_3_triangulation_3/include/CGAL/Periodic_3_regular_triangulation_3.h $
+// $Id: Periodic_3_regular_triangulation_3.h 8c60160 %aI Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@inria.fr>
@@ -479,8 +479,6 @@ public:
       is_large_point_set = false;
 
     std::vector<Weighted_point> points(first, last);
-    CGAL::cpp98::random_shuffle(points.begin(), points.end());
-    Cell_handle hint;
     std::vector<Vertex_handle> dummy_points_vhs, double_vertices;
     std::vector<Weighted_point> dummy_points;
     typename std::vector<Weighted_point>::iterator pbegin = points.begin();
@@ -493,6 +491,8 @@ public:
     }
     else
     {
+      CGAL::cpp98::random_shuffle(points.begin(), points.end());
+      pbegin = points.begin();
       while(!is_1_cover())
       {
         insert(*pbegin);
@@ -502,6 +502,8 @@ public:
       }
     }
 
+    CGAL_postcondition(is_1_cover());
+
     // Spatial sorting can only be applied to bare points, so we need an adaptor
     typedef typename Geom_traits::Construct_point_3 Construct_point_3;
     typedef typename boost::result_of<const Construct_point_3(const Weighted_point&)>::type Ret;
@@ -513,6 +515,7 @@ public:
                    CGAL::internal::boost_::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
                        geom_traits().construct_point_3_object()), geom_traits()));
 
+    Cell_handle hint;
     Conflict_tester tester(*pbegin, this);
     Point_hider hider(this);
     Cover_manager cover_manager(*this);
