@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/detect_features.h $
+// $Id: detect_features.h d940632 %aI Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -97,14 +97,16 @@ template <typename PatchIdMap, typename Handle_type, typename Int>
 typename PatchIdMapWrapper<PatchIdMap, Int>::value_type
 get(PatchIdMapWrapper<PatchIdMap, Int>& map, Handle_type h)
 {
-  return get(map.map, h) - map.offset;
+  typedef typename PatchIdMapWrapper<PatchIdMap, Int>::value_type value_type;
+  return value_type(get(map.map, h) - map.offset);
 }
 
 template <typename PatchIdMap, typename Handle_type, typename Int>
 void put(PatchIdMapWrapper<PatchIdMap, Int>& map, Handle_type h,
          typename PatchIdMapWrapper<PatchIdMap, Int>::value_type pid)
 {
-  put(map.map, h, pid + map.offset);
+  typedef typename PatchIdMapWrapper<PatchIdMap, Int>::value_type value_type;
+  put(map.map, h, value_type(pid + map.offset));
 }
 
 
@@ -127,14 +129,14 @@ template <typename PatchIdMap, typename Handle_type, typename Int>
 typename PatchIdMapWrapper<PatchIdMap, std::pair<Int, Int> >::value_type
 get(PatchIdMapWrapper<PatchIdMap, std::pair<Int, Int> >& map, Handle_type h)
 {
-  return get(map.map, h).first - map.offset;
+  return Int(get(map.map, h).first - map.offset);
 }
 
 template <typename PatchIdMap, typename Handle_type, typename Int>
 void put(PatchIdMapWrapper<PatchIdMap, std::pair<Int, Int> >& map, Handle_type h,
          typename PatchIdMapWrapper<PatchIdMap, std::pair<Int, Int> >::value_type pid)
 {
-  put(map.map, h, std::pair<Int, Int>(pid+map.offset, 0));
+  put(map.map, h, std::pair<Int, Int>(Int(pid+map.offset), 0));
 }
 
 template <typename PolygonMesh, typename PatchIdMap,
@@ -258,7 +260,7 @@ template<typename GT,
  * \param pmesh the polygon mesh
  * \param angle_in_deg the dihedral angle bound
  * \param edge_is_feature_map the property map that will contain the sharp-or-not status of each edge of `pmesh`
- * \param np optional \ref pmp_namedparameters "Named Parameters" described below
+ * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
@@ -398,7 +400,7 @@ namespace internal
  * \param angle_in_deg the dihedral angle bound
  * \param edge_is_feature_map the property map that will contain the sharp-or-not status of each edge of `pmesh`
  * \param patch_id_map the property map that will contain the surface patch ids for the faces of `pmesh`.
- * \param np optional \ref pmp_namedparameters "Named Parameters" described below
+ * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
