@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Three/include/CGAL/Three/Polyhedron_demo_plugin_interface.h $
+// $Id: Polyhedron_demo_plugin_interface.h ee57fc2 %aI Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Laurent RINEAU
@@ -37,23 +38,28 @@ namespace CGAL {
 namespace Three {
 class Scene_interface;
   /*!
-   * This class gives some virtual functions to help making a plugin
+   * This virtual class provides the basic functions used for making a plugin.
    */
 class Polyhedron_demo_plugin_interface 
 {
 public:
-  //! Initializes the plugin and gives access to a Message interface, that
-  //! can be used to display messages in the console dock widget.
-  //! @see init(QMainWindow*, CGAL::Three::Scene_interface*)
+  //! \brief Initializes the plugin
+  //! This function acts like a constructor. This is where the attributes must be initialized.
+  //! The Message_interface allows to print warnings or errors on the screen and the `Console` widget.
   virtual void init(QMainWindow* , CGAL::Three::Scene_interface* , Messages_interface*) = 0;
 
-  //! Decides if the plugin's actions will be displayed or not.
-  //! @returns \c true, if the plugin is applicable, \c false
+  //! \brief Indicates if an action is usable or not.
+  //! This function usually tests the type of the selected item to determine if `action` can be applied to it,
+  //! but not necessarly.
+  //! @returns \c true if `action` can be called in the current state, \c false
   //! otherwise
-  virtual bool applicable(QAction*) const = 0;
-  //!The list of the actions of the plugin.
+  virtual bool applicable(QAction* action) const = 0;
+  //!Contains all the plugin's actions.
   virtual QList<QAction*> actions() const = 0;
-  //!If you need to do something when the plugin is closed, do it here
+  //!\brief Is called when the application is closed.
+  //! Override this function if you need to perform a specific action
+  //! when the application is closed, like hide the widgets if you don't want
+  //! their visibility to be saved.
   virtual void closure() {
  }
 protected :
