@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/STL_Extension/include/CGAL/vector.h $
+// $Id: vector.h f0c1d3a %aI Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Andreas Fabri <Andreas.Fabri@sophia.inria.fr>
@@ -65,7 +66,7 @@ public:
 
     // Allows construction of const_iterator from iterator
     template < class A, class B, class C>
-    vector_iterator( const vector_iterator<A,B,C>& i) : ptr( &*i) {}
+    vector_iterator( const vector_iterator<A,B,C>& i) : ptr(i.operator->()) {}
 
     // OPERATIONS Forward Category
     // ---------------------------
@@ -157,13 +158,22 @@ public:
     // Note: the standard requires the following types to be equivalent
     // to T, T*, const T*, T&, const T&, size_t, and ptrdiff_t, respectively.
     // So we don't pass these types to the iterators explicitly.
+#ifdef CGAL_CXX11
+  typedef typename std::allocator_traits<Allocator>::value_type            value_type;
+  typedef typename std::allocator_traits<Allocator>::pointer               pointer;
+  typedef typename std::allocator_traits<Allocator>::const_pointer         const_pointer;
+  typedef typename std::allocator_traits<Allocator>::size_type             size_type;
+  typedef typename std::allocator_traits<Allocator>::difference_type       difference_type;
+#else
     typedef typename Allocator::value_type           value_type;
     typedef typename Allocator::pointer              pointer;
     typedef typename Allocator::const_pointer        const_pointer;
-    typedef typename Allocator::reference            reference;
-    typedef typename Allocator::const_reference      const_reference;
     typedef typename Allocator::size_type            size_type;
     typedef typename Allocator::difference_type      difference_type;
+#endif
+
+    typedef value_type&                              reference;
+    typedef const value_type&                        const_reference;
     typedef std::random_access_iterator_tag          iterator_category;
     typedef vector_iterator< T, reference, pointer>  iterator;
     typedef vector_iterator< T, const_reference, const_pointer>
@@ -182,8 +192,22 @@ protected:
     iterator end_of_storage;
 
     // ALLOCATION AND CONSTRUCTION HELPERS
-    void construct( iterator i, const T& x) { alloc.construct( &*i, x);}
-    void destroy( iterator i) { alloc.destroy( &*i); }
+    void construct( iterator i, const T& x) {
+#ifdef CGAL_CXX11
+      std::allocator_traits<Allocator>::construct(alloc,&*i, x);
+#else
+      alloc.construct(&*i, x);
+#endif
+    }
+  
+    void destroy( iterator i) {
+#ifdef CGAL_CXX11
+      std::allocator_traits<Allocator>::destroy(alloc,&*i);
+#else
+      alloc.destroy( &*i);
+#endif
+    }
+  
     void destroy( iterator first, iterator last) {
         // destroy in reverse order than construction
         while ( last != first) {
